###
### Copyright 2002 Ximian, Inc.
###
### This program is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License, version 2,
### as published by the Free Software Foundation.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
###

import sys
import os
import os.path
import glob
from ConfigParser import ConfigParser
import string
from yum.packages import YumInstalledPackage
import rumtalk
import rumformat
import rumcommand
from yum.packages import YumLocalPackage


class CleanCmd(rumcommand.RumCommand):

    def name(self):
        return "clean"

    def aliases(self):
        return ["cl"]

    def is_basic(self):
        return 1

    def category(self):
        return "system"

    def arguments(self):
        return ""

    def description_short(self):
        return "Clean cache"

    def local_opt_table(self):
        return [["", "headers",  "", "Clean headers"],
                ["", "packages", "", "Clean packages"],
                ["", "metadata", "", "Clean metadata"],
                ["", "cache",    "", "Clean cache"],
                ["", "dbcache",  "", "Clean dbcache"],
                ["", "all",      "", "Clean all (default)"]]

    def execute(self, options_dict, non_option_args):
        yum = self.yum()
        
        if options_dict.has_key('all') or len(options_dict.values()) == 0:
            yum.cleanPackages()
            yum.cleanHeaders()
            yum.cleanMetadata()
            yum.cleanSqlite()
            rumtalk.message("--- Successfully cleaned all ---")
        elif options_dict.has_key('headers'):
            yum.cleanHeaders()
            rumtalk.message("--- Successfully cleaned headers ---")
        elif options_dict.has_key('packages'):
            yum.cleanPackages()
            rumtalk.message("--- Successfully cleaned packages ---")
        elif options_dict.has_key('metadata'):
            yum.cleanMetadata()
            rumtalk.message("--- Successfully cleaned metadata ---")
        elif options_dict.has_key('dbcache'):
            yum.cleanSqlite()
            rumtalk.message("--- Successfully cleaned dbcache ---")
        else:
            rumtalk.error("Unknown argument '%s'" % non_option_args[0])

        yum.plugins.run('clean')

rumcommand.register(CleanCmd)


class RefreshCmd(rumcommand.RumCommand):

    def name(self):
        return "refresh"

    def aliases(self):
        return ["ref"]

    def is_basic(self):
        return 1

    def category(self):
        return "system"

    def arguments(self):
        return "[repo id] [repo id] ..."

    def description_short(self):
        return "Refresh the package cache"

    def execute(self, options_dict, non_option_args):
        yum = self.yum()
        repos = []

        if non_option_args:
            for a in non_option_args:
                tmp = yum.repos.findRepos(a)
                if not tmp:
                    rumtalk.error("Repository '%s' not found." % a)
                repos = repos + tmp
        else:
            repos = yum.repos.findRepos('*')

        for repo in repos:
            repo.metadata_expire = 0

        yum.doRepoSetup()
        yum.repos.populateSack(with='metadata', cacheonly=1)
        yum.repos.populateSack(with='filelists', cacheonly=1)
        yum.repos.populateSack(with='otherdata', cacheonly=1)
        rumtalk.message('--- Refresh Successful ---')


rumcommand.register(RefreshCmd)

class RepolistCmd(rumcommand.RumCommand):

    def name(self):
        return "repos"

    def aliases(self):
        return ["sl", "ca", "ch"]

    def is_basic(self):
        return 1

    def category(self):
        return "system"

    def description_short(self):
        return "List active repositories"

    def local_opt_table(self):
        return [["", "enabled-only", "", "List enabled services only"]]

    def execute(self, options_dict, non_option_args):
        yum = self.yum(repos=False)

        enabled_only = options_dict.has_key("enabled-only")
        repos = []
        for repo in yum.repos.findRepos('*'):
            if enabled_only:
                if repo.isEnabled():
                    repos.append((repo.id, repo.name))
            else:
                repos.append((repo.id, repo.name, rumformat.bool_to_str(repo.isEnabled())))

        repos.sort(lambda x,y:cmp(x[0], y[0]))

        if len(repos):
            headers = ['ID', 'Name']
            if not enabled_only:
                headers.append('Enabled')
            rumformat.tabular(headers, repos)
        else:
            rumtalk.message("--- No repositories found ---")

rumcommand.register(RepolistCmd)

class RepoCmd(rumcommand.RumCommand):

    def name(self):
        return "enable"

    def aliases(self):
        return ["en"]

    def is_basic(self):
        return 1

    def category(self):
        return "system"

    def description_short(self):
        return "Enable a repository"

    def is_hidden(self):
        return 1

    def find_repo_file(self, id):
        yum = self.yum(repos=False)
        
        for repodir in yum.conf.reposdir:
            for repo in glob.glob(os.path.join(repodir, '*.repo')):
                parser = ConfigParser()
                parser.read(repo)

                if id in parser.sections():
                    return repo

        return None

    def setenabled(self, repos, val):
        yum = self.yum(repos=False)

        for repodir in yum.conf.reposdir:
            for repo in glob.glob(os.path.join(repodir, '*.repo')):
                parser = ConfigParser()
                parser.read(repo)
                for section in parser.sections():
                    if section in repos:
                        parser.set(section, 'enabled', str(val))
                        parser.write(file(repo, 'w'))

                        if val:
                            rumtalk.message("--- Enabled repository '%s' ---" % section)
                        else:
                            rumtalk.message("--- Disabled repository '%s' ---" % section)

class RepoEnableCmd(RepoCmd):

    def name(self):
        return "enable"

    def aliases(self):
        return ["en"]

    def is_basic(self):
        return 1

    def is_hidden(self):
        return 0

    def category(self):
        return "system"

    def description_short(self):
        return "Enable a repository"

    def execute(self, options_dict, non_option_args):
        self.setenabled(non_option_args, 1)

rumcommand.register(RepoEnableCmd)

class RepoDisableCmd(RepoCmd):

    def name(self):
        return "disable"

    def aliases(self):
        return ["di"]

    def is_basic(self):
        return 1

    def is_hidden(self):
        return 0

    def category(self):
        return "system"

    def description_short(self):
        return "Disable a repository"

    def execute(self, options_dict, non_option_args):
        self.setenabled(non_option_args, 0)

rumcommand.register(RepoDisableCmd)

class RepoAddCmd(RepoCmd):

    def name(self):
        return "repo-add"

    def aliases(self):
        return ["sa", "ra"]

    def is_basic(self):
        return 1

    def is_hidden(self):
        return 0

    def category(self):
        return "system"

    def arguments(self):
        return "<name> <url>"

    def description_short(self):
        return "Add a repository"

    def local_opt_table(self):
        return [["c", "check-signatures",  "", "Check gpg signatures"],
                ["n", "name", "name", "Name of the repository"],
                ["m", "mirrorlist", "", "The url specified is a mirror list"]]

    def check_url(self, url):
        if url.startswith('http://') or url.startswith('ftp://') or \
           url.startswith('file://') or url.startswith('https://'):
            return True
        else:
            return False
        

    def execute(self, options_dict, non_option_args):
        if len(non_option_args) != 2:
            self.usage()
            return False

        repoid = non_option_args[0]
        url = non_option_args[1]

        if not self.check_url(url):
            rumtalk.error("Invalid url '%s'" % url)
            return False

        if self.find_repo_file(repoid) != None:
            rumtalk.error("Repository '%s' already exists" % repoid)
            return False

        yum = self.yum(repos=False)

        repopath = os.path.join(yum.conf.reposdir[0], repoid + ".repo")
        if not os.path.exists(os.path.dirname(repopath)):
            os.makedirs(os.path.dirname(repopath))
        
        parser = ConfigParser()
        parser.add_section(repoid)

        name = repoid
        if options_dict.has_key('name'):
            name = options_dict['name']
            
        parser.set(repoid, "name", name)

        if options_dict.has_key('mirrorlist'):
            parser.set(repoid, "mirrorlist", url)
        else:
            parser.set(repoid, "baseurl", url)
            
        parser.set(repoid, "enabled", "1")

        gpgval = "0"
        if options_dict.has_key('check-signatures'):
            gpgval = "1"
            
        parser.set(repoid, "gpgcheck", gpgval)

        
        
        parser.write(file(repopath, "w+"))

        rumtalk.message("--- Successfully added '%s' ---" % repoid)


rumcommand.register(RepoAddCmd)

class RepoDelCmd(RepoCmd):

    def name(self):
        return "repo-delete"

    def aliases(self):
        return ["sd", "rd"]

    def is_basic(self):
        return 1

    def is_hidden(self):
        return 0

    def category(self):
        return "system"

    def arguments(self):
        return "<id>"

    def description_short(self):
        return "Remove a repository"

    def execute(self, options_dict, non_option_args):
        if len(non_option_args) != 1:
            self.usage()
            return False

        repoid = non_option_args[0]
        repopath = self.find_repo_file(repoid)
        if repopath == None:
            rumtalk.error("Repository '%s' does not exist" % repoid)
            return False

        parser = ConfigParser()
        parser.read(repopath)
        parser.remove_section(repoid)
        if len(parser.sections()) == 0:
            os.unlink(repopath)
        else:
            parser.write(file(repopath, 'w+'))

        rumtalk.message("--- Successfully removed '%s' ---" % repoid)


rumcommand.register(RepoDelCmd)
